@@ -323,8 +323,6 @@ NB_MODULE(nanovdb, m)
         .value("Vec4d", GridType::Vec4d)
         .value("Index", GridType::Index)
         .value("OnIndex", GridType::OnIndex)
-        .value("IndexMask", GridType::IndexMask)
-        .value("OnIndexMask", GridType::OnIndexMask)
         .value("PointIndex", GridType::PointIndex)
         .value("Vec3u8", GridType::Vec3u8)
         .value("Vec3u16", GridType::Vec3u16)
