@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_GRID_HAS_BEEN_INCLUDED
 #define OPENVDB_GRID_HAS_BEEN_INCLUDED
@@ -10,9 +10,9 @@
 #include "io/io.h"
 #include "math/Transform.h"
 #include "tree/Tree.h"
+#include "util/Assert.h"
 #include "util/logging.h"
 #include "util/Name.h"
-#include <cassert>
 #include <iostream>
 #include <set>
 #include <type_traits>
@@ -173,10 +173,8 @@ public:
     /// shared with other grids.  The pointer is guaranteed to be non-null.
     virtual TreeBase::ConstPtr constBaseTreePtr() const = 0;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
     /// @brief Return true if tree is not shared with another grid.
     virtual bool isTreeUnique() const = 0;
-#endif
 
     /// @brief Return a reference to this grid's tree, which might be
     /// shared with other grids.
@@ -588,10 +586,10 @@ public:
     using ValueAllIter        = typename _TreeType::ValueAllIter;
     using ValueAllCIter       = typename _TreeType::ValueAllCIter;
 
-    using Accessor            = typename tree::ValueAccessor<_TreeType, true>;
-    using ConstAccessor       = typename tree::ValueAccessor<const _TreeType, true>;
-    using UnsafeAccessor      = typename tree::ValueAccessor<_TreeType, false>;
-    using ConstUnsafeAccessor = typename tree::ValueAccessor<const _TreeType, false>;
+    using Accessor            = typename _TreeType::Accessor;
+    using ConstAccessor       = typename _TreeType::ConstAccessor;
+    using UnsafeAccessor      = typename _TreeType::UnsafeAccessor;
+    using ConstUnsafeAccessor = typename _TreeType::ConstUnsafeAccessor;
 
     /// @brief ValueConverter<T>::Type is the type of a grid having the same
     /// hierarchy as this grid but a different value type, T.
@@ -731,7 +729,7 @@ public:
     /// @brief Return an accessor that provides random read and write access
     /// to this grid's voxels.
     /// @details The accessor is safe in the sense that it is registered with this grid's tree.
-    Accessor getAccessor() { return Accessor(tree()); }
+    Accessor getAccessor() { return mTree->getAccessor(); }
     /// @brief Return an unsafe accessor that provides random read and write access
     /// to this grid's voxels.
     /// @details The accessor is unsafe in the sense that it is not registered
@@ -739,11 +737,11 @@ public:
     /// over a registered accessor, but it is unsafe if the tree topology is modified.
     /// @warning Only use this method if you're an expert and know the
     /// risks of using an unregistered accessor (see tree/ValueAccessor.h)
-    UnsafeAccessor getUnsafeAccessor() { return UnsafeAccessor(tree()); }
+    UnsafeAccessor getUnsafeAccessor() { return mTree->getUnsafeAccessor(); }
     /// Return an accessor that provides random read-only access to this grid's voxels.
-    ConstAccessor getAccessor() const { return ConstAccessor(tree()); }
+    ConstAccessor getAccessor() const { return mTree->getConstAccessor(); }
     /// Return an accessor that provides random read-only access to this grid's voxels.
-    ConstAccessor getConstAccessor() const { return ConstAccessor(tree()); }
+    ConstAccessor getConstAccessor() const { return mTree->getConstAccessor(); }
     /// @brief Return an unsafe accessor that provides random read-only access
     /// to this grid's voxels.
     /// @details The accessor is unsafe in the sense that it is not registered
@@ -751,7 +749,7 @@ public:
     /// over a registered accessor, but it is unsafe if the tree topology is modified.
     /// @warning Only use this method if you're an expert and know the
     /// risks of using an unregistered accessor (see tree/ValueAccessor.h)
-    ConstUnsafeAccessor getConstUnsafeAccessor() const { return ConstUnsafeAccessor(tree()); }
+    ConstUnsafeAccessor getConstUnsafeAccessor() const { return mTree->getConstUnsafeAccessor(); }
 
     /// Return an iterator over all of this grid's active values (tile and voxel).
     ValueOnIter   beginValueOn()       { return tree().beginValueOn(); }
@@ -875,10 +873,6 @@ public:
     CoordBBox evalActiveVoxelBoundingBox() const override;
     /// Return the dimensions of the axis-aligned bounding box of all active voxels.
     Coord evalActiveVoxelDim() const override;
-    /// Return the minimum and maximum active values in this grid.
-    OPENVDB_DEPRECATED_MESSAGE("Switch from grid->evalMinMax(minVal, maxVal) to \
-tools::minMax(grid->tree()). Use threaded = false for serial execution")
-    void evalMinMax(ValueType& minVal, ValueType& maxVal) const;
 
     /// Return the number of bytes of memory used by this grid.
     /// @todo Add transform().memUsage()
@@ -900,11 +894,8 @@ tools::minMax(grid->tree()). Use threaded = false for serial execution")
     //@}
     /// @brief Return true if tree is not shared with another grid.
     /// @note This is a virtual function with ABI=8
-#if OPENVDB_ABI_VERSION_NUMBER >= 8
     bool isTreeUnique() const final;
-#else
-    bool isTreeUnique() const;
-#endif
+
     //@{
     /// @brief Return a reference to this grid's tree, which might be
     /// shared with other grids.
@@ -1068,7 +1059,7 @@ struct TreeAdapter
     using TreePtrType          = typename TreeType::Ptr;
     using ConstTreePtrType     = typename TreeType::ConstPtr;
     using NonConstTreePtrType  = typename NonConstTreeType::Ptr;
-    using GridType             = Grid<TreeType>;
+    using GridType             = Grid<NonConstTreeType>;
     using NonConstGridType     = Grid<NonConstTreeType>;
     using GridPtrType          = typename GridType::Ptr;
     using NonConstGridPtrType  = typename NonConstGridType::Ptr;
@@ -1078,14 +1069,14 @@ struct TreeAdapter
     using ConstAccessorType    = typename tree::ValueAccessor<const TreeType>;
     using NonConstAccessorType = typename tree::ValueAccessor<NonConstTreeType>;
 
-    static TreeType& tree(TreeType& t) { return t; }
-    static TreeType& tree(GridType& g) { return g.tree(); }
-    static const TreeType& tree(const TreeType& t) { return t; }
-    static const TreeType& tree(const GridType& g) { return g.tree(); }
-    static const TreeType& constTree(TreeType& t) { return t; }
-    static const TreeType& constTree(GridType& g) { return g.constTree(); }
-    static const TreeType& constTree(const TreeType& t) { return t; }
-    static const TreeType& constTree(const GridType& g) { return g.constTree(); }
+    static NonConstTreeType& tree(NonConstTreeType& t) { return t; }
+    static NonConstTreeType& tree(NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& tree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& tree(const NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& constTree(NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(NonConstGridType& g) { return g.constTree(); }
+    static const NonConstTreeType& constTree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(const NonConstGridType& g) { return g.constTree(); }
 };
 
 
@@ -1108,19 +1099,19 @@ struct TreeAdapter<Grid<_TreeType> >
     using ConstAccessorType    = typename tree::ValueAccessor<const TreeType>;
     using NonConstAccessorType = typename tree::ValueAccessor<NonConstTreeType>;
 
-    static TreeType& tree(TreeType& t) { return t; }
-    static TreeType& tree(GridType& g) { return g.tree(); }
-    static const TreeType& tree(const TreeType& t) { return t; }
-    static const TreeType& tree(const GridType& g) { return g.tree(); }
-    static const TreeType& constTree(TreeType& t) { return t; }
-    static const TreeType& constTree(GridType& g) { return g.constTree(); }
-    static const TreeType& constTree(const TreeType& t) { return t; }
-    static const TreeType& constTree(const GridType& g) { return g.constTree(); }
+    static NonConstTreeType& tree(NonConstTreeType& t) { return t; }
+    static NonConstTreeType& tree(NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& tree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& tree(const NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& constTree(NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(NonConstGridType& g) { return g.constTree(); }
+    static const NonConstTreeType& constTree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(const NonConstGridType& g) { return g.constTree(); }
 };
 
-/// Partial specialization for ValueAccessor types
+/// Partial specialization for const Grid types
 template<typename _TreeType>
-struct TreeAdapter<tree::ValueAccessor<_TreeType> >
+struct TreeAdapter<const Grid<_TreeType> >
 {
     using TreeType             = _TreeType;
     using NonConstTreeType     = typename std::remove_const<TreeType>::type;
@@ -1137,16 +1128,51 @@ struct TreeAdapter<tree::ValueAccessor<_TreeType> >
     using ConstAccessorType    = typename tree::ValueAccessor<const TreeType>;
     using NonConstAccessorType = typename tree::ValueAccessor<NonConstTreeType>;
 
-    static TreeType& tree(TreeType& t) { return t; }
-    static TreeType& tree(GridType& g) { return g.tree(); }
-    static TreeType& tree(AccessorType& a) { return a.tree(); }
-    static const TreeType& tree(const TreeType& t) { return t; }
-    static const TreeType& tree(const GridType& g) { return g.tree(); }
-    static const TreeType& tree(const AccessorType& a) { return a.tree(); }
-    static const TreeType& constTree(TreeType& t) { return t; }
-    static const TreeType& constTree(GridType& g) { return g.constTree(); }
-    static const TreeType& constTree(const TreeType& t) { return t; }
-    static const TreeType& constTree(const GridType& g) { return g.constTree(); }
+    static NonConstTreeType& tree(NonConstTreeType& t) { return t; }
+    static NonConstTreeType& tree(NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& tree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& tree(const NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& constTree(NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(NonConstGridType& g) { return g.constTree(); }
+    static const NonConstTreeType& constTree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(const NonConstGridType& g) { return g.constTree(); }
+};
+
+/// Partial specialization for ValueAccessor types
+template<typename _TreeType>
+struct TreeAdapter<tree::ValueAccessor<_TreeType> >
+{
+    using TreeType             = _TreeType;
+    using NonConstTreeType     = typename std::remove_const<TreeType>::type;
+    using TreePtrType          = typename TreeType::Ptr;
+    using ConstTreePtrType     = typename TreeType::ConstPtr;
+    using NonConstTreePtrType  = typename NonConstTreeType::Ptr;
+    using GridType             = Grid<NonConstTreeType>;
+    using NonConstGridType     = Grid<NonConstTreeType>;
+    using GridPtrType          = typename GridType::Ptr;
+    using NonConstGridPtrType  = typename NonConstGridType::Ptr;
+    using ConstGridPtrType     = typename GridType::ConstPtr;
+    using ValueType            = typename TreeType::ValueType;
+    using AccessorType         = typename tree::ValueAccessor<TreeType>;
+    using ConstAccessorType    = typename tree::ValueAccessor<const NonConstTreeType>;
+    using NonConstAccessorType = typename tree::ValueAccessor<NonConstTreeType>;
+
+    static NonConstTreeType& tree(NonConstTreeType& t) { return t; }
+    static NonConstTreeType& tree(NonConstGridType& g) { return g.tree(); }
+    static NonConstTreeType& tree(NonConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& tree(ConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& tree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& tree(const NonConstGridType& g) { return g.tree(); }
+    static const NonConstTreeType& tree(const NonConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& tree(const ConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& constTree(NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(NonConstGridType& g) { return g.constTree(); }
+    static const NonConstTreeType& constTree(NonConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& constTree(ConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& constTree(const NonConstTreeType& t) { return t; }
+    static const NonConstTreeType& constTree(const NonConstGridType& g) { return g.constTree(); }
+    static const NonConstTreeType& constTree(const NonConstAccessorType& a) { return a.tree(); }
+    static const NonConstTreeType& constTree(const ConstAccessorType& a) { return a.tree(); }
 };
 
 //@}
@@ -1553,16 +1579,6 @@ Grid<TreeT>::topologyDifference(const Grid<OtherTreeType>& other)
 ////////////////////////////////////////
 
 
-template<typename TreeT>
-inline void
-Grid<TreeT>::evalMinMax(ValueType& minVal, ValueType& maxVal) const
-{
-    OPENVDB_NO_DEPRECATION_WARNING_BEGIN
-    tree().evalMinMax(minVal, maxVal);
-    OPENVDB_NO_DEPRECATION_WARNING_END
-}
-
-
 template<typename TreeT>
 inline CoordBBox
 Grid<TreeT>::evalActiveVoxelBoundingBox() const
@@ -1609,13 +1625,15 @@ template<typename TreeT>
 inline void
 Grid<TreeT>::readBuffers(std::istream& is)
 {
-    if (!hasMultiPassIO() || (io::getFormatVersion(is) < OPENVDB_FILE_VERSION_MULTIPASS_IO)) {
+    io::checkFormatVersion(is);
+
+    if (!hasMultiPassIO()) {
         tree().readBuffers(is, saveFloatAsHalf());
     } else {
         uint16_t numPasses = 1;
         is.read(reinterpret_cast<char*>(&numPasses), sizeof(uint16_t));
         const io::StreamMetadata::Ptr meta = io::getStreamMetadataPtr(is);
-        assert(bool(meta));
+        OPENVDB_ASSERT(bool(meta));
         for (uint16_t passIndex = 0; passIndex < numPasses; ++passIndex) {
             uint32_t pass = (uint32_t(numPasses) << 16) | uint32_t(passIndex);
             meta->setPass(pass);
@@ -1631,13 +1649,15 @@ template<typename TreeT>
 inline void
 Grid<TreeT>::readBuffers(std::istream& is, const CoordBBox& bbox)
 {
-    if (!hasMultiPassIO() || (io::getFormatVersion(is) < OPENVDB_FILE_VERSION_MULTIPASS_IO)) {
+    io::checkFormatVersion(is);
+
+    if (!hasMultiPassIO()) {
         tree().readBuffers(is, bbox, saveFloatAsHalf());
     } else {
         uint16_t numPasses = 1;
         is.read(reinterpret_cast<char*>(&numPasses), sizeof(uint16_t));
         const io::StreamMetadata::Ptr meta = io::getStreamMetadataPtr(is);
-        assert(bool(meta));
+        OPENVDB_ASSERT(bool(meta));
         for (uint16_t passIndex = 0; passIndex < numPasses; ++passIndex) {
             uint32_t pass = (uint32_t(numPasses) << 16) | uint32_t(passIndex);
             meta->setPass(pass);
@@ -1667,7 +1687,7 @@ Grid<TreeT>::writeBuffers(std::ostream& os) const
     } else {
         // Determine how many leaf buffer passes are required for this grid
         const io::StreamMetadata::Ptr meta = io::getStreamMetadataPtr(os);
-        assert(bool(meta));
+        OPENVDB_ASSERT(bool(meta));
         uint16_t numPasses = 1;
         meta->setCountingPasses(true);
         meta->setPass(0);
@@ -1754,7 +1774,7 @@ createLevelSet(Real voxelSize, Real halfWidth)
     using ValueType = typename GridType::ValueType;
 
     // GridType::ValueType is required to be a floating-point scalar.
-    static_assert(std::is_floating_point<ValueType>::value,
+    static_assert(openvdb::is_floating_point<ValueType>::value,
         "level-set grids must be floating-point-valued");
 
     typename GridType::Ptr grid = GridType::create(
