@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #ifndef OPENVDB_TYPES_HAS_BEEN_INCLUDED
 #define OPENVDB_TYPES_HAS_BEEN_INCLUDED
@@ -8,27 +8,7 @@
 #include "Platform.h"
 #include "TypeList.h" // backwards compat
 
-#ifdef OPENVDB_USE_IMATH_HALF
-#ifdef OPENVDB_IMATH_VERSION
-#include <Imath/half.h>
-#else
-#include <OpenEXR/half.h>
-#endif
-namespace openvdb {
-OPENVDB_USE_VERSION_NAMESPACE
-namespace OPENVDB_VERSION_NAME {
-namespace math {
-using half = half;
-}}}
-#else
-#include <openvdb/math/Half.h>
-namespace openvdb {
-OPENVDB_USE_VERSION_NAMESPACE
-namespace OPENVDB_VERSION_NAME {
-namespace math {
-using half = internal::half;
-}}}
-#endif
+#include <openvdb/math/HalfDecl.h>
 
 #include <openvdb/math/Math.h>
 #include <openvdb/math/BBox.h>
@@ -58,12 +38,13 @@ using Int64   = int64_t;
 using Int     = Int32;
 using Byte    = unsigned char;
 using Real    = double;
+using Half    = math::half;
 
 // Two-dimensional vector types
 using Vec2R = math::Vec2<Real>;
 using Vec2I = math::Vec2<Index32>;
 using Vec2f = math::Vec2<float>;
-using Vec2H = math::Vec2<math::half>;
+using Vec2H = math::Vec2<Half>;
 using math::Vec2i;
 using math::Vec2s;
 using math::Vec2d;
@@ -72,7 +53,7 @@ using math::Vec2d;
 using Vec3R = math::Vec3<Real>;
 using Vec3I = math::Vec3<Index32>;
 using Vec3f = math::Vec3<float>;
-using Vec3H = math::Vec3<math::half>;
+using Vec3H = math::Vec3<Half>;
 using Vec3U8 = math::Vec3<uint8_t>;
 using Vec3U16 = math::Vec3<uint16_t>;
 using math::Vec3i;
@@ -87,7 +68,7 @@ using BBoxd = math::BBox<Vec3d>;
 using Vec4R = math::Vec4<Real>;
 using Vec4I = math::Vec4<Index32>;
 using Vec4f = math::Vec4<float>;
-using Vec4H = math::Vec4<math::half>;
+using Vec4H = math::Vec4<Half>;
 using math::Vec4i;
 using math::Vec4s;
 using math::Vec4d;
@@ -182,6 +163,77 @@ using PointDataIndex32 = PointIndex<Index32, 1>;
 using PointDataIndex64 = PointIndex<Index64, 1>;
 
 
+////////////////////////////////////////
+
+/// @brief  Macros to help determine whether or not a class has a particular
+///   member function.
+/// @details  These macros work by instantiating unique templated instances
+///   of helper structs for a particular member function signature and name
+///   which can then be queried.
+///     - The first macro, INVOKABLE, defines a helper struct which determines
+///       if a member function can be called with a given set of argument types.
+///       Note that the return type is not provided.
+///     - The second macro defines a helper struct which determines if the
+///       member function exists with the _exact_ same signature, including
+///       all argument and function attributes (const-ness, noexcept, etc)
+///
+///   Use the first solution if all you want to determine is whether a given
+///   method can be called, and the second if you need exact resolution.
+/// @code
+///   // example class
+///   struct MyClass { int test(double) { return 0; } };
+///
+///   // The following examples work from the struct type created by:
+///   OPENVDB_INIT_INVOKABLE_MEMBER_FUNCTION(test);
+///
+///   // Will assert true!
+///   static_assert(OPENVDB_HAS_INVOKABLE_MEMBER_FUNCTION(MyClass, test, double));
+///   // Will assert true, int can be converted to double
+///   static_assert(OPENVDB_HAS_INVOKABLE_MEMBER_FUNCTION(MyClass, test, int));
+///   // Will assert false, needs at least one argument
+///   static_assert(OPENVDB_HAS_INVOKABLE_MEMBER_FUNCTION(MyClass, test);
+///
+///
+///   // The following examples work from the struct type created by:
+///   OPENVDB_INIT_MEMBER_FUNCTION(test);
+///
+///   // Will assert fail
+///   static_assert(OPENVDB_HAS_MEMBER_FUNCTION(MyClass, test, void(MyClass::*)(double)));
+///   // Only case where this assert true
+///   static_assert(OPENVDB_HAS_MEMBER_FUNCTION(MyClass, test, int(MyClass::*)(double)));
+///
+/// @endcode
+#define OPENVDB_INIT_INVOKABLE_MEMBER_FUNCTION(F)              \
+    template <typename ClassT, typename... Args>               \
+    struct HasInvokableMemberFunction_##F {                    \
+    private:                                                   \
+        template <typename T>                                  \
+        static auto check(T*) ->                               \
+            decltype(std::declval<T>().                        \
+                F(std::declval<Args>()...), std::true_type()); \
+        template <typename T>                                  \
+        static auto check(...) -> std::false_type;             \
+    public:                                                    \
+        static constexpr bool value =                          \
+            decltype(check<ClassT>(nullptr))::value;           \
+    };
+#define OPENVDB_HAS_INVOKABLE_MEMBER_FUNCTION(T, F, ...) \
+    HasInvokableMemberFunction_##F<T, __VA_ARGS__>::value
+
+#define OPENVDB_INIT_MEMBER_FUNCTION(F)                           \
+    template<typename ClassT, typename Signature>                 \
+    struct HasMemberFunction_##F {                                \
+        template <typename U, U> struct sigmatch;                 \
+        template <typename U> static std::true_type               \
+            check(sigmatch<Signature, &U::F>*);                   \
+        template <typename>   static std::false_type check(...);  \
+        static const bool value = std::is_same<std::true_type,    \
+            decltype(check<ClassT>(nullptr))>::value;             \
+    };
+#define OPENVDB_HAS_MEMBER_FUNCTION(T, F, S) \
+    HasMemberFunction_##F<T, S>::value
+
+
 ////////////////////////////////////////
 
 
@@ -198,6 +250,45 @@ struct IsSpecializationOf<Template<Args...>, Template>: public std::true_type {}
 ////////////////////////////////////////
 
 
+/// @brief  Re-implementation of C++17's index_sequence and the helper alias
+///   make_index_sequence. This was introduced to fix an issue with clang's
+///   builtin implementation which treats template specializations of builtin
+///   templates differently when a subsequent parameter is dependent. The
+///   result is a resolution failure during partial specialization selection.
+///   For example, the following will fail to specialize:
+///
+/// @code
+///    struct Test { static const int VALUE = 1; };
+///
+///    template <typename T, typename S = std::make_index_sequence<T::VALUE>>
+///    struct Item {};
+///    template <typename T> struct Adapter {};
+///    template <typename T> struct Adapter<Item<T>> {};  // FAIL: will never be selected.
+/// @endcode
+///
+///  This is fixed from Clang16. See also:
+///    https://reviews.llvm.org/D133262
+///    https://github.com/llvm/llvm-project/issues/42102
+///    https://github.com/llvm/llvm-project/issues/51928
+///    https://github.com/llvm/llvm-project/commit/f4ea3bd4b2086e6de10131b197aaf7d066a24df8
+template <std::size_t... Ns>
+struct index_sequence {};
+
+template <std::size_t N, std::size_t... Is>
+auto make_index_sequence_impl() {
+    // only one branch is considered. The other may be ill-formed
+    if constexpr (N == 0) return index_sequence<Is...>(); // end case
+    else return make_index_sequence_impl<N-1, N-1, Is...>(); // recursion
+}
+
+template <std::size_t N>
+using make_index_sequence =
+    std::decay_t<decltype(make_index_sequence_impl<N>())>;
+
+
+////////////////////////////////////////
+
+
 template<typename T, bool = IsSpecializationOf<T, math::Vec2>::value ||
                             IsSpecializationOf<T, math::Vec3>::value ||
                             IsSpecializationOf<T, math::Vec4>::value>
@@ -407,6 +498,21 @@ template<typename FromType, typename ToType> struct CopyConstness<const FromType
 /// @endcond
 
 
+////////////////////////////////////////
+template<class T>
+struct is_floating_point : std::is_floating_point<T> { };
+
+template<>
+struct is_floating_point<Half> : std::is_floating_point<float> { };
+
+
+template<class T>
+struct is_signed : std::is_signed<T> { };
+
+template<>
+struct is_signed<Half> : std::is_signed<float> { };
+
+
 ////////////////////////////////////////
 
 
@@ -649,6 +755,23 @@ class Steal {};
 /// @brief Tag dispatch class that distinguishes constructors during file input
 class PartialCreate {};
 
+// For half compilation
+namespace math {
+template<>
+inline auto cwiseAdd(const Vec3H& v, const float s)
+{
+    Vec3H out;
+    const Half* ip = v.asPointer();
+    Half* op = out.asPointer();
+    for (unsigned i = 0; i < 3; ++i, ++op, ++ip) {
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_BEGIN
+        *op = *ip + s;
+        OPENVDB_NO_TYPE_CONVERSION_WARNING_END
+    }
+    return out;
+}
+} // namespace math
+
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
