@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb/openvdb.h>
 
@@ -19,7 +19,7 @@
 #include <openvdb/util/CpuTimer.h>
 #endif
 
-#include "gtest/gtest.h"
+#include <gtest/gtest.h>
 
 using namespace openvdb;
 using namespace openvdb::points;
@@ -1794,9 +1794,10 @@ struct HaltOnSecondInterrupt : public util::NullInterrupter
     inline bool wasInterrupted(int percent = -1) override
     {
         (void)percent;
-        if (mInterrupt)     return true;
-        mInterrupt = true;
-        return false;
+        bool expected = false;
+        // Atomically: if mInterrupt is false, set it to true and return false
+        //             if mInterrupt is true, return true
+        return !mInterrupt.compare_exchange_strong(expected, true);
     }
     std::atomic<bool> mInterrupt{false};
 };
@@ -2153,7 +2154,9 @@ TEST_F(TestPointRasterizeFrustum, testStreaming)
 
     auto leaf = points->tree().cbeginLeaf();
     EXPECT_TRUE(leaf);
+#ifdef OPENVDB_USE_DELAYED_LOADING
     EXPECT_TRUE(leaf->buffer().isOutOfCore());
+#endif
 
     using Rasterizer = FrustumRasterizer<PointDataGrid>;
     using Settings = FrustumRasterizerSettings;
@@ -2324,10 +2327,12 @@ TEST_F(TestPointRasterizeFrustum, testStreaming)
     auto points2 = points->deepCopy();
     points2->setTransform(transform);
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     // verify both grids are out-of-core
 
     EXPECT_TRUE(points->tree().cbeginLeaf()->buffer().isOutOfCore());
     EXPECT_TRUE(points2->tree().cbeginLeaf()->buffer().isOutOfCore());
+#endif
 
 #ifndef ONLY_RASTER_FLOAT
     // memory tests
