@@ -1,10 +1,9 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file unittest/TestPoissonSolver.cc
 /// @authors D.J. Hill, Peter Cucka
 
-#include "gtest/gtest.h"
 #include <openvdb/openvdb.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/ConjGradient.h> // for JacobiPreconditioner
@@ -12,8 +11,10 @@
 #include <openvdb/tools/LevelSetSphere.h> // for tools::createLevelSetSphere()
 #include <openvdb/tools/LevelSetUtil.h> // for tools::sdfToFogVolume()
 #include <openvdb/tools/MeshToVolume.h> // for createLevelSetBox()
-#include <openvdb/tools/Morphology.h> // for tools::erodeVoxels()
 #include <openvdb/tools/PoissonSolver.h>
+
+#include <gtest/gtest.h>
+
 #include <cmath>
 
 
