@@ -8,37 +8,23 @@
 #include <openvdb/points/PointDataGrid.h>
 #include <openvdb/points/PointConversion.h>
 
-#include <cppunit/extensions/HelperMacros.h>
+#include <gtest/gtest.h>
 
-class TestAXRun : public CppUnit::TestCase
+class TestAXRun : public ::testing::Test
 {
-public:
-
-    CPPUNIT_TEST_SUITE(TestAXRun);
-    CPPUNIT_TEST(singleRun);
-    CPPUNIT_TEST(multiRun);
-    CPPUNIT_TEST(regressions);
-    CPPUNIT_TEST_SUITE_END();
-
-    void singleRun();
-    void multiRun();
-    void regressions();
 };
 
-CPPUNIT_TEST_SUITE_REGISTRATION(TestAXRun);
-
-void
-TestAXRun::singleRun()
+TEST_F(TestAXRun, singleRun)
 {
     openvdb::FloatGrid f;
     f.setName("a");
     f.tree().setValueOn({0,0,0}, 0.0f);
 
     openvdb::ax::run("@a = 1.0f;", f);
-    CPPUNIT_ASSERT_EQUAL(1.0f, f.tree().getValue({0,0,0}));
+    ASSERT_EQ(1.0f, f.tree().getValue({0,0,0}));
 
     openvdb::ax::run("@b = 2.0f;", f, {{"b", "a"}});
-    CPPUNIT_ASSERT_EQUAL(2.0f, f.tree().getValue({0,0,0}));
+    ASSERT_EQ(2.0f, f.tree().getValue({0,0,0}));
     openvdb::math::Transform::Ptr defaultTransform =
         openvdb::math::Transform::createLinearTransform();
     const std::vector<openvdb::Vec3d> singlePointZero = {openvdb::Vec3d::zero()};
@@ -50,20 +36,19 @@ TestAXRun::singleRun()
     const auto leafIter = points->tree().cbeginLeaf();
     const auto& descriptor = leafIter->attributeSet().descriptor();
 
-    CPPUNIT_ASSERT_EQUAL(size_t(2), descriptor.size());
+    ASSERT_EQ(size_t(2), descriptor.size());
     const size_t idx = descriptor.find("a");
-    CPPUNIT_ASSERT(idx != openvdb::points::AttributeSet::INVALID_POS);
-    CPPUNIT_ASSERT(descriptor.valueType(idx) == openvdb::typeNameAsString<float>());
+    ASSERT_TRUE(idx != openvdb::points::AttributeSet::INVALID_POS);
+    ASSERT_TRUE(descriptor.valueType(idx) == openvdb::typeNameAsString<float>());
     openvdb::points::AttributeHandle<float> handle(leafIter->constAttributeArray(idx));
-    CPPUNIT_ASSERT_EQUAL(1.0f, handle.get(0));
+    ASSERT_EQ(1.0f, handle.get(0));
 
     openvdb::ax::run("@b = 2.0f;", *points, {{"b","a"}});
 
-    CPPUNIT_ASSERT_EQUAL(2.0f, handle.get(0));
+    ASSERT_EQ(2.0f, handle.get(0));
 }
 
-void
-TestAXRun::multiRun()
+TEST_F(TestAXRun, multiRun)
 {
     {
         // multi volumes
@@ -76,12 +61,12 @@ TestAXRun::multiRun()
         std::vector<openvdb::GridBase::Ptr> v { f1, f2 };
 
         openvdb::ax::run("@a = @b = 1;", v);
-        CPPUNIT_ASSERT_EQUAL(1.0f, f1->tree().getValue({0,0,0}));
-        CPPUNIT_ASSERT_EQUAL(1.0f, f2->tree().getValue({0,0,0}));
+        ASSERT_EQ(1.0f, f1->tree().getValue({0,0,0}));
+        ASSERT_EQ(1.0f, f2->tree().getValue({0,0,0}));
 
         openvdb::ax::run("@c = @d = 2;", v, {{"c","a"}, {"d","b"}});
-        CPPUNIT_ASSERT_EQUAL(2.0f, f1->tree().getValue({0,0,0}));
-        CPPUNIT_ASSERT_EQUAL(2.0f, f2->tree().getValue({0,0,0}));
+        ASSERT_EQ(2.0f, f1->tree().getValue({0,0,0}));
+        ASSERT_EQ(2.0f, f2->tree().getValue({0,0,0}));
     }
 
     {
@@ -104,45 +89,44 @@ TestAXRun::multiRun()
         const auto& descriptor1 = leafIter1->attributeSet().descriptor();
         const auto& descriptor2 = leafIter1->attributeSet().descriptor();
 
-        CPPUNIT_ASSERT_EQUAL(size_t(3), descriptor1.size());
-        CPPUNIT_ASSERT_EQUAL(size_t(3), descriptor2.size());
+        ASSERT_EQ(size_t(3), descriptor1.size());
+        ASSERT_EQ(size_t(3), descriptor2.size());
         const size_t idx1 = descriptor1.find("a");
-        CPPUNIT_ASSERT_EQUAL(idx1, descriptor2.find("a"));
+        ASSERT_EQ(idx1, descriptor2.find("a"));
         const size_t idx2 = descriptor1.find("b");
-        CPPUNIT_ASSERT_EQUAL(idx2, descriptor2.find("b"));
-        CPPUNIT_ASSERT(idx1 != openvdb::points::AttributeSet::INVALID_POS);
-        CPPUNIT_ASSERT(idx2 != openvdb::points::AttributeSet::INVALID_POS);
+        ASSERT_EQ(idx2, descriptor2.find("b"));
+        ASSERT_TRUE(idx1 != openvdb::points::AttributeSet::INVALID_POS);
+        ASSERT_TRUE(idx2 != openvdb::points::AttributeSet::INVALID_POS);
 
-        CPPUNIT_ASSERT(descriptor1.valueType(idx1) == openvdb::typeNameAsString<float>());
-        CPPUNIT_ASSERT(descriptor1.valueType(idx2) == openvdb::typeNameAsString<float>());
-        CPPUNIT_ASSERT(descriptor2.valueType(idx1) == openvdb::typeNameAsString<float>());
-        CPPUNIT_ASSERT(descriptor2.valueType(idx2) == openvdb::typeNameAsString<float>());
+        ASSERT_TRUE(descriptor1.valueType(idx1) == openvdb::typeNameAsString<float>());
+        ASSERT_TRUE(descriptor1.valueType(idx2) == openvdb::typeNameAsString<float>());
+        ASSERT_TRUE(descriptor2.valueType(idx1) == openvdb::typeNameAsString<float>());
+        ASSERT_TRUE(descriptor2.valueType(idx2) == openvdb::typeNameAsString<float>());
 
         openvdb::points::AttributeHandle<float> handle(leafIter1->constAttributeArray(idx1));
-        CPPUNIT_ASSERT_EQUAL(1.0f, handle.get(0));
+        ASSERT_EQ(1.0f, handle.get(0));
         handle = openvdb::points::AttributeHandle<float>(leafIter1->constAttributeArray(idx2));
-        CPPUNIT_ASSERT_EQUAL(1.0f, handle.get(0));
+        ASSERT_EQ(1.0f, handle.get(0));
 
         handle = openvdb::points::AttributeHandle<float>(leafIter2->constAttributeArray(idx1));
-        CPPUNIT_ASSERT_EQUAL(1.0f, handle.get(0));
+        ASSERT_EQ(1.0f, handle.get(0));
         handle = openvdb::points::AttributeHandle<float>(leafIter2->constAttributeArray(idx2));
-        CPPUNIT_ASSERT_EQUAL(1.0f, handle.get(0));
+        ASSERT_EQ(1.0f, handle.get(0));
 
         openvdb::ax::run("@c = @d = 2;", v, {{"c","a"}, {"d","b"}});
         handle = openvdb::points::AttributeHandle<float>(leafIter1->constAttributeArray(idx1));
-        CPPUNIT_ASSERT_EQUAL(2.0f, handle.get(0));
+        ASSERT_EQ(2.0f, handle.get(0));
         handle = openvdb::points::AttributeHandle<float>(leafIter1->constAttributeArray(idx2));
-        CPPUNIT_ASSERT_EQUAL(2.0f, handle.get(0));
+        ASSERT_EQ(2.0f, handle.get(0));
 
         handle = openvdb::points::AttributeHandle<float>(leafIter2->constAttributeArray(idx1));
-        CPPUNIT_ASSERT_EQUAL(2.0f, handle.get(0));
+        ASSERT_EQ(2.0f, handle.get(0));
         handle = openvdb::points::AttributeHandle<float>(leafIter2->constAttributeArray(idx2));
-        CPPUNIT_ASSERT_EQUAL(2.0f, handle.get(0));
+        ASSERT_EQ(2.0f, handle.get(0));
     }
 }
 
-void
-TestAXRun::regressions()
+TEST_F(TestAXRun, regressions)
 {
     openvdb::points::PointDataGrid::Ptr p1(new openvdb::points::PointDataGrid);
     openvdb::points::PointDataGrid::Ptr p2(new openvdb::points::PointDataGrid);
@@ -155,22 +139,22 @@ TestAXRun::regressions()
         // test error on points and volumes
         std::vector<openvdb::GridBase::Ptr> v1 { f1, p1 };
         std::vector<openvdb::GridBase::Ptr> v2 { p1, f1 };
-        CPPUNIT_ASSERT_THROW(openvdb::ax::run("@a = 1.0f;", v1), openvdb::AXCompilerError);
-        CPPUNIT_ASSERT_THROW(openvdb::ax::run("@a = 1.0f;", v2), openvdb::AXCompilerError);
+        ASSERT_THROW(openvdb::ax::run("@a = 1.0f;", v1), openvdb::AXCompilerError);
+        ASSERT_THROW(openvdb::ax::run("@a = 1.0f;", v2), openvdb::AXCompilerError);
     }
 
     // Various tests which have been caught during developement
 
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("{} =", g1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("{} =", g2), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("{} =", *f1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("{} =", *p1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("@c = 1.0f", g1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("@c = 1.0f", g2), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("@c = 1.0f", *f1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("@c = 1.0f", *p1), openvdb::AXSyntaxError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("if (v@v) {}", g1), openvdb::AXCompilerError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("if (v@v) {}", g2), openvdb::AXCompilerError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("if (v@v) {}", *f1), openvdb::AXCompilerError);
-    CPPUNIT_ASSERT_THROW(openvdb::ax::run("if (v@v) {}", *p1), openvdb::AXCompilerError);
+    ASSERT_THROW(openvdb::ax::run("{} =", g1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("{} =", g2), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("{} =", *f1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("{} =", *p1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("@c = 1.0f", g1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("@c = 1.0f", g2), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("@c = 1.0f", *f1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("@c = 1.0f", *p1), openvdb::AXSyntaxError);
+    ASSERT_THROW(openvdb::ax::run("if (v@v) {}", g1), openvdb::AXCompilerError);
+    ASSERT_THROW(openvdb::ax::run("if (v@v) {}", g2), openvdb::AXCompilerError);
+    ASSERT_THROW(openvdb::ax::run("if (v@v) {}", *f1), openvdb::AXCompilerError);
+    ASSERT_THROW(openvdb::ax::run("if (v@v) {}", *p1), openvdb::AXCompilerError);
 }
