@@ -1,13 +1,14 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 #include <openvdb_ax/ast/AST.h>
 #include <openvdb_ax/ast/Scanners.h>
 #include <openvdb_ax/ast/PrintTree.h>
 #include <openvdb_ax/Exceptions.h>
-#include <openvdb_ax/test/util.h>
 
-#include <cppunit/extensions/HelperMacros.h>
+#include "../util.h"
+
+#include <gtest/gtest.h>
 
 #include <string>
 
@@ -307,34 +308,27 @@ static const unittest_util::CodeTests tests =
 
 }
 
-class TestLoopNode : public CppUnit::TestCase
+class TestLoopNode : public ::testing::Test
 {
-    public:
-
-    CPPUNIT_TEST_SUITE(TestLoopNode);
-    CPPUNIT_TEST(testSyntax);
-    CPPUNIT_TEST(testASTNode);
-    CPPUNIT_TEST_SUITE_END();
-
-    void testSyntax() { TEST_SYNTAX_PASSES(tests); }
-    void testASTNode();
 };
 
-CPPUNIT_TEST_SUITE_REGISTRATION(TestLoopNode);
+TEST_F(TestLoopNode, testSyntax)
+{
+    TEST_SYNTAX_PASSES(tests);
+}
 
-void TestLoopNode::testASTNode()
+TEST_F(TestLoopNode, testASTNode)
 {
     for (const auto& test : tests) {
         const std::string& code = test.first;
         const Node* expected = test.second.get();
         const Tree::ConstPtr tree = parse(code.c_str());
-        CPPUNIT_ASSERT_MESSAGE(ERROR_MSG("No AST returned", code), static_cast<bool>(tree));
+        ASSERT_TRUE(static_cast<bool>(tree)) << ERROR_MSG("No AST returned", code);
 
         // get the first statement
         const Node* result = tree->child(0)->child(0);
-        CPPUNIT_ASSERT(result);
-        CPPUNIT_ASSERT_MESSAGE(ERROR_MSG("Invalid AST node", code),
-            Node::LoopNode == result->nodetype());
+        ASSERT_TRUE(result);
+        ASSERT_TRUE(Node::LoopNode == result->nodetype()) << ERROR_MSG("Invalid AST node", code);
 
         std::vector<const Node*> resultList, expectedList;
         linearize(*result, resultList);
@@ -346,7 +340,7 @@ void TestLoopNode::testASTNode()
             openvdb::ax::ast::print(*expected, true, os);
             os << "Result:\n";
             openvdb::ax::ast::print(*result, true, os);
-            CPPUNIT_FAIL(ERROR_MSG("Mismatching Trees for Loop code", code) + os.str());
+            FAIL() << ERROR_MSG("Mismatching Trees for Loop code", code) + os.str();
         }
     }
 }
