@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 //
 /// @file SOP_NodeVDB.cc
 /// @author FX R&D OpenVDB team
@@ -10,8 +10,8 @@
 #include <openvdb/points/PointDataGrid.h>
 #include "PointUtils.h"
 #include "Utils.h"
-#include "GEO_PrimVDB.h"
-#include "GU_PrimVDB.h"
+#include <GEO/GEO_PrimVDB.h>
+#include <GU/GU_PrimVDB.h>
 #include <GU/GU_Detail.h>
 #include <GU/GU_PrimPoly.h>
 #include <OP/OP_NodeInfoParms.h>
@@ -350,107 +350,6 @@ SOP_NodeVDB::getNodeSpecificInfoText(OP_Context &context, OP_NodeInfoParms &parm
 ////////////////////////////////////////
 
 
-OP_ERROR
-SOP_NodeVDB::duplicateSourceStealable(const unsigned index,
-    OP_Context& context, GU_Detail **pgdp, GU_DetailHandle& gdh, bool clean)
-{
-    OPENVDB_NO_DEPRECATION_WARNING_BEGIN
-    // traverse upstream nodes, if unload is not possible, duplicate the source
-    if (!isSourceStealable(index, context)) {
-        duplicateSource(index, context, *pgdp, clean);
-        unlockInput(index);
-        return error();
-    }
-    OPENVDB_NO_DEPRECATION_WARNING_END
-
-    // get the input GU_Detail handle and unlock the inputs
-    GU_DetailHandle inputgdh = inputGeoHandle(index);
-
-    unlockInput(index);
-    SOP_Node *input = CAST_SOPNODE(getInput(index));
-
-    if (!input) {
-        addError(SOP_MESSAGE, "Invalid input SOP Node when attempting to unload.");
-        return error();
-    }
-
-    // explicitly unload the data from the input SOP
-    const bool unloadSuccessful = input->unloadData();
-
-    // check if we only have one reference
-    const bool soleReference = (inputgdh.getRefCount() == 1);
-
-    // if the unload was unsuccessful or the reference count is not one, we fall back to
-    // explicitly copying the input onto the gdp
-    if (!(unloadSuccessful && soleReference)) {
-        const GU_Detail *src = inputgdh.readLock();
-        UT_ASSERT(src);
-        if (src)  (*pgdp)->copy(*src);
-        inputgdh.unlock(src);
-        return error();
-    }
-
-    // release our old write lock on gdp (setup by cookMe())
-    gdh.unlock(*pgdp);
-    // point to the input's old gdp and setup a write lock
-    gdh = inputgdh;
-    *pgdp = gdh.writeLock();
-
-    return error();
-}
-
-
-bool
-SOP_NodeVDB::isSourceStealable(const unsigned index, OP_Context& context) const
-{
-    struct Local {
-        static inline OP_Node* nextStealableInput(
-            const unsigned idx, const fpreal now, const OP_Node* node)
-        {
-            OP_Node* input = node->getInput(idx);
-            while (input) {
-                OP_Node* passThrough = input->getPassThroughNode(now);
-                if (!passThrough) break;
-                input = passThrough;
-            }
-            return input;
-        }
-    }; // struct Local
-
-    const fpreal now = context.getTime();
-
-    for (OP_Node* node = Local::nextStealableInput(index, now, this); node != nullptr;
-        node = Local::nextStealableInput(index, now, node))
-    {
-        // cont'd if it is a SOP_NULL.
-        std::string opname = node->getName().toStdString().substr(0, 4);
-        if (opname == "null") continue;
-
-        // if the SOP is a cache SOP we don't want to try and alter its data without a deep copy
-        if (dynamic_cast<SOP_Cache*>(node))  return false;
-
-        if (node->getUnload() != 0)
-            return true;
-        else
-            return false;
-    }
-    return false;
-}
-
-
-OP_ERROR
-SOP_NodeVDB::duplicateSourceStealable(const unsigned index, OP_Context& context)
-{
-    OPENVDB_NO_DEPRECATION_WARNING_BEGIN
-    auto error = this->duplicateSourceStealable(index, context, &gdp, myGdpHandle, true);
-    OPENVDB_NO_DEPRECATION_WARNING_END
-    return error;
-}
-
-
-////////////////////////////////////////
-
-
 const SOP_NodeVerb*
 SOP_NodeVDB::cookVerb() const
 {
