@@ -59,7 +59,11 @@ public:
 
     // Return true for a given input if the connector to the input
     // should be drawn dashed rather than solid.
-    int isRefInput(unsigned idx) const override { return (idx == 1); }
+    int isRefInput(OP_InputIdx idx) const override
+    {
+        UT_ASSERT(idx >= 0);
+        return (idx == 1);
+    }
 
 protected:
     bool updateParmsFlags() override;
@@ -1255,7 +1259,7 @@ SOP_OpenVDB_Convert::Cache::referenceMeshing(
 
     // Transfer Primitive Attributes
     if (!boss.wasInterrupted() && transferAttributes && refGeo && indexGrid) {
-        hvdb::transferPrimitiveAttributes(*refGeo, *gdp, *indexGrid, boss.interrupter(), surfaceGroup);
+        hvdb::transferPrimitiveAttributes(*refGeo, *gdp, *indexGrid, boss, surfaceGroup);
     }
 
 
